"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
}
Object.defineProperty(exports, "__esModule", { value: true });
const koa_compose_1 = __importDefault(require("koa-compose"));
const socket_1 = __importDefault(require("../socket"));
const events_1 = require("events");
describe('Socket', () => {
    let mockWebSocket;
    let socket;
    let handler;
    let steps = [];
    let handlerPromise;
    let listeners;
    beforeEach(async () => {
        jest.useRealTimers();
        mockWebSocket = new events_1.EventEmitter();
        steps = [];
        handlerPromise = new Promise(resolve => {
            handler = jest.fn(async (ctx, next) => {
                setTimeout(() => {
                    steps.push(`[${ctx.event}]`);
                    resolve();
                }, 10);
                return handlerPromise;
            });
            listeners = new Map();
            listeners.set('message', [handler]);
            listeners.set('close', [handler]);
        });
    });
    describe('Socket is created with one middleware', () => {
        let middleware;
        let middlewarePromise;
        beforeEach(async () => {
            const firstMiddleware = async (ctx, next) => {
                steps.push('[before]');
                await next(ctx);
                steps.push('[after]');
            };
            middlewarePromise = new Promise(resolve => {
                middleware = jest.fn(koa_compose_1.default([(ctx, next) => firstMiddleware(ctx, next).then(resolve)]));
            });
            socket = new socket_1.default(mockWebSocket, null, listeners, middleware);
            socket.update(listeners, middleware);
        });
        it('invokes the handler in correct order', async () => {
            mockWebSocket.emit('message', 'message');
            await middlewarePromise;
            expect(steps).toEqual(['[before]', '[message]', '[after]']);
        });
        it('invokes only the handler and not middleware for close event', async () => {
            mockWebSocket.emit('close', 'message');
            await handlerPromise;
            expect(middleware).not.toHaveBeenCalled();
            expect(handler).toHaveBeenCalledTimes(1);
            expect(steps).toEqual(['[close]']);
        });
    });
    describe('Socket is created with no middleware', () => {
        beforeEach(async () => {
            socket = new socket_1.default(mockWebSocket, null, listeners, koa_compose_1.default([]));
        });
        it('invokes the handler correctly', async () => {
            mockWebSocket.emit('message', 'message');
            await handlerPromise;
            expect(steps).toEqual(['[message]']);
        });
    });
});
//# sourceMappingURL=socket.spec.js.map