/// <reference types="node" />
/// <reference types="ws" />
/// <reference types="koa-compose" />
/// <reference types="koa" />
import * as http from 'http';
import { TlsOptions } from 'tls';
import * as Koa from 'koa';
import compose from 'koa-compose';
import * as WebSocket from 'ws';
import Socket from './socket';
export interface SocketContext {
    data: any;
    event: string;
    request: http.ServerRequest;
    socket: WebSocket;
}
export declare type EventHandler = (ctx: SocketContext, data: any) => any;
export declare type Middleware = (ctx: SocketContext, next: () => Promise<any>) => any;
export declare class WsSocket {
    middleware: Middleware[];
    composed: compose.ComposedMiddleware<SocketContext>;
    connections: Map<string, Socket>;
    listeners: Map<string, EventHandler[]>;
    wss: WebSocket.Server | null;
    attach(app: Koa, https?: boolean, tlsOpts?: TlsOptions): void;
    onConnection: (ws: WebSocket, request: http.IncomingMessage) => void;
    /**
     * Pushes a middleware on to the stack
     * @param fn <Function> the middleware function to execute
     */
    use(fn: Middleware): this;
    /**
     * Adds a new listener to the stack
     * @param event <String> the event id
     * @param handler <Function> the callback to execute
     * @return this
     */
    on(event: string, handler: EventHandler): this;
    /**
     * Removes a listener from the event
     * @param event <String> if omitted will remove all listeners
     * @param handler <Function> if omitted will remove all from the event
     * @return this
     */
    off(event: string, handler: EventHandler): this;
    /**
     * Updates all existing connections with current listeners and middleware
     * @private
     */
    updateConnections(): void;
}
