"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Socket {
    constructor(ws, request, listeners, middleware) {
        this.ws = ws;
        this.request = request;
        this.middleware = middleware;
        this.update(listeners, middleware);
    }
    update(listeners, middleware) {
        this.ws.removeAllListeners();
        this.middleware = middleware;
        listeners.forEach((handlers, event) => {
            if (event === 'connection') {
                return;
            }
            handlers.forEach(handler => {
                this.ws.on(event, async (message) => {
                    const ctx = {
                        event,
                        data: message,
                        request: this.request,
                        socket: this.ws,
                    };
                    if (event === 'message') {
                        await this.middleware(ctx, () => handler(ctx, message));
                    }
                    else {
                        await handler(ctx, message);
                    }
                });
            });
        });
    }
}
exports.default = Socket;
//# sourceMappingURL=socket.js.map